/**
 * Created by John on 3/3/2016.
 *
 *
 * USES excel4node
 *
 */
var json2xl = require('./json2xl');
var moment = require('moment');
var fs = require("fs");
var winston = require('winston');
var logger = winston.loggers.get('default');

/**
 *
 * @param params { data:{}, fields:{} }
 * @param callback(err, file)
 */
function fromJson2Xl(params) {
    //if (!callback || typeof callback !== 'function') {
    //    throw new Error('Callback is required');
    //}

    return new Promise(function (resolve, reject) {
        checkParams(params).then(function (good) {

                var m = moment().format("YYYYMMDD_hhmm");
                var filepath = "components/public/uploads/"; //no longer used to generate the file
                var filePrefix = (params.prefix) ? params.prefix : "excel";
                var sheetName = filePrefix + "_" + m;
                var fileName = sheetName + '.xlsx';

                var wbOpts = {
                    jszip: {
                        compression: 'DEFLATE'
                    }
                };

                var wsOpts = {
                    margins: {
                        left: .75,
                        right: .75,
                        top: 1.0,
                        bottom: 1.0,
                        footer: .5,
                        header: .5
                    },
                    //printOptions:{
                    //    centerHorizontal : true,
                    //    centerVertical : false
                    //},
                    view: {
                        zoom: 100
                    },
                    fitToPage: {
                        fitToHeight: 100,
                        orientation: 'landscape',
                    }
                };

                var xlData = {
                    "worksheets": [sheetName],
                    "filepath": filepath,
                    "filename": fileName,
                    "rows": [],
                    "config": {
                        "wbOpts": wbOpts,
                        "wsOpts": wsOpts
                    }
                };

                var hdrRow = []; //array of cells
                for (var f = 0; f < params.fields.length; f++) {
                    hdrRow.push(buildHdrCell(params.fields[f]))
                }
                xlData.rows.push(hdrRow);

                for (var dr = 0; dr < params.data.length; dr++) {
                    //for each value in the row
                    var newRow = [];
                    //each row is an object of all fields in the row
                    for (var f = 0; f < params.fields.length; f++) {
                        //if ( params.fields[f] === "default_vendors") {
                        //    newRow.push(buildHdrCell(params.data[dr][params.fields[f]], 'string'));
                        //} else {
                        newRow.push(buildDataCell(params.data[dr][params.fields[f]], 'string'));
                        //}
                    }
                    xlData.rows.push(newRow);
                }
                var response = json2xl.Json2XL(xlData);
                if (response.file) {
                    resolve(response.file);
                } else {
                    reject(response);
                }
            },
            function (bad) {
                reject("Bad params to Excel exporter");
            });
    });
}

function buildHdrCell(value) {
    var cell = {
        "value": (value) ? String(value) : "",
        "dataType": "string",
        "style": [ {"fontWeight": "bold"}]
    }
    return cell;
}

function buildDataCell(value, type) {
    var cell = {
        "value": (value) ? String(value) : "",
        "dataType": (type) ? type : "string",
        "style": [ {"fontFamily": "Arial"}]
    }
    return cell;
}


/**
 * Check passing params
 *
 * @param {Object} params Function parameters containing data, fields,
 * delimiter, default value, mark quotes and hasCSVColumnTitle
 * @param {Function} callback Callback function returning error when invalid field is found
 */
function checkParams(params) {
    // if data is an Object, not in array [{}], then just create 1 item array.
    // So from now all data in array of object format.
    return new Promise(function (resolve, reject) {
        if (!Array.isArray(params.data)) {
            var ar = [];
            ar[0] = params.data;
            params.data = ar;
        }

        if (!params.fields && params.data && params.data.length) {
            var firstData = params.data[0];

            if (typeof firstData === 'object' && firstData !== null) {
                params.fields = Object.keys(firstData);
            } else {
                logger.error('params should be a valid object.');
                reject(false);
            }
        }

        //#check fieldNames
        if (params.fieldNames && params.fieldNames.length !== params.fields.length) {
            logger.error('fieldNames and fields should be of the same length, if fieldNames is provided.');
            reject(false);
        }

        params.fieldNames = params.fieldNames || params.fields;

        //#check delimiter
        params.del = params.del || ',';

        //#check end of line character
        params.eol = params.eol || '';

        //#check quotation mark
        params.quotes = typeof params.quotes === 'string' ? params.quotes : '"';

        //#check nested option
        params.nested = params.nested || false;

        //#check default value
        params.defaultValue = params.defaultValue;

        //#check hasCSVColumnTitle, if it is not explicitly set to false then true.
        if (params.hasCSVColumnTitle !== false) {
            params.hasCSVColumnTitle = true;
        }

        resolve(true);
    });
}


module.exports = {
    fromJson2Xl: fromJson2Xl
}