/**
 * Created by John on 4/27/2017.
 */
"use strict";
var moment = require('moment');

module.exports = function(sequelize, DataTypes) {
    var User = require("./user")(sequelize, DataTypes);
    var Message = sequelize.define('Message', {
            id: {
                type: DataTypes.INTEGER,
                field: 'id',
                primaryKey: true,
                autoIncrement: true
            },
            message: {
                type: DataTypes.STRING,
                field: 'msg'
            },
            timestamp: {
                type: DataTypes.DATE,
                field: 'date_created',
                set: function(val) {
                    if (val == null) {
                        this.setDataValue('timestamp', new Date());
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('timestamp', mom);
                                } else {
                                    this.setDataValue('timestamp', m)
                                }
                            } else {
                                this.setDataValue('timestamp', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('timestamp', mo);
                        }
                    }
                }
            },
            createdByUserId: {
                type: DataTypes.INTEGER,
                field: 'created_by_user_id'
            },
            keepAliveStart: {
                type: DataTypes.DATE,
                field: 'keep_alive_start',
                set: function(val) {
                    if (val == null) {
                        this.setDataValue('keepAliveStart', null);
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('keepAliveStart', mom);
                                } else {
                                    this.setDataValue('keepAliveStart', m)
                                }
                            } else {
                                this.setDataValue('keepAliveStart', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('keepAliveStart', mo);
                        }
                    }
                },
                get: function() {
                    var dt = this.getDataValue('keepAliveStart');
                    if (dt !== null && dt.__proto__.isValid && !dt.isValid()) {
                        this.setDataValue('keepAliveStart', null);
                    }
                    return this.getDataValue('keepAliveStart');
                }
            },
            keepAliveEnd: {
                type: DataTypes.DATE,
                field: 'keep_alive_end',
                set: function(val) {
                    if (val == null) {
                        this.setDataValue('keepAliveEnd', null);
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('keepAliveEnd', mom);
                                } else {
                                    this.setDataValue('keepAliveEnd', m)
                                }
                            } else {
                                this.setDataValue('keepAliveEnd', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('keepAliveEnd', mo);
                        }
                    }
                },
                get: function() {
                    var dt = this.getDataValue('keepAliveEnd');
                    if (dt !== null && dt.__proto__.isValid && !dt.isValid()) {
                        this.setDataValue('keepAliveEnd', null);
                    }
                    return this.getDataValue('keepAliveEnd');
                }
            }
        },
        {
            timestamps: true,
            createdAt: 'timestamp',
            updatedAt: false,
            deletedAt: false,
            tableName: 'tbl_messages',
            getterMethods   : {
                recipientsAsString       : function()  {
                    if (this.recipients) {
                        if (Array.isArray(this.recipients)) {
                            var out = "";

                            for (var r = 0; r < this.recipients.length; r++) {
                                var cat = this.recipients[r];
                                if (cat.clinic)
                                    out += ((r === 0) ? "" : ", ") + cat.clinic.clinicId;
                                else
                                    out += ((r === 0) ? "" : ", ") + cat.clinicId;
                            }

                            return out;
                        }
                        else
                            return this.recipients.clinic.clinicId;
                    } else {
                        return "";
                    }
                },
                recurrenceType : function() {
                    if (this.keepAliveStart && this.keepAliveEnd) {
                        var out = "recurring";
                        var dt = new Date(this.keepAliveEnd);
                        if (dt < (new Date())) {
                            out = "expired";
                        }
                        return out;
                    }
                    return "one-time";
                }
            }
        }
    );
    Message.belongsTo(User, {as: "createdBy", foreignKey: 'createdByUserId'});
    var MsgRecipient = require("./msgRecipient")(sequelize, DataTypes);
    Message.hasMany(MsgRecipient, {as: "recipients", foreignKey: 'messageId'});
    return Message;
};
