/**
 * Created by will on 10/13/15.
 */

var config = angular.module('app.config', [
    'ui.router'
]);
config.config(['$urlRouterProvider', '$locationProvider', '$httpProvider', '$stateProvider', function($urlRouterProvider, $locationProvider, $httpProvider, $stateProvider) {
    var checkLoggedIn = function($q, $timeout, $http, $location, $rootScope) {
        $http.get('/api/loggedin/').success(function(user){
            if (user && user !== '0') {
                if (user.isActive) {
                    $rootScope.user = user;
                    return $q.when();
                } else {
                    return forceRedirectLogin('Please contact an administrator.', $q, $timeout, $location, $rootScope);
                }
            }
            else {
                return forceRedirectLogin('You need to log in.', $q, $timeout, $location, $rootScope);
            }
        });
    };

    function forceRedirectLogin(msg, $q, $timeout, $location, $rootScope) {
        $rootScope.message = msg;
        $rootScope.user = null;
        $timeout(function(){
            $location.url('/login');
        });
        return $q.reject();
    }

    $httpProvider.interceptors.push(function($q, $location) {
        return function(promise) {
            return promise.then(
                // Success: just return the response
                function(response){
                    return response;
                },
                // Error: check the error status to get only the 401
                function(response) {
                    if (response.status === 401)
                        $location.url('/login');
                    return $q.reject(response);
                }
            );
        }
    });

    $stateProvider
        .state('login', {
            url: '/login',
            templateUrl: 'login/login.html',
            controller: 'LoginCtrl',
            loginRequired: false
        })
        .state('root', {
            url: '/',
            templateUrl: 'root.html',
//            controller: 'BaseCtrl',
            abstract: true
        })
        .state('root.countdata', {
            url: 'countdata',
            templateUrl: 'countData/countData.html',
            controller: 'CountDataCtrl',
            resolve: {
                loggedin: checkLoggedIn
            },
            loginRequired: true
        })
        .state('root.masteritem', {
            url: 'masteritem',
            templateUrl: 'masterItem/masterItem.html',
            controller: 'MasterItemCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.clinicprofile', {
            url: 'clinicprofile',
            templateUrl: 'clinicProfile/clinicProfile.html',
            controller: 'ClinicProfileCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.inventory', {
            url: 'inventory',
            templateUrl: 'inventory/inventory.html',
            controller: 'InventoryCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.receipt', {
            url: 'receipt',
            templateUrl: 'receipt/receipt.html',
            controller: 'ReceiptCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.consumption', {
            url: 'consumption',
            templateUrl: 'consumption/consumption.html',
            controller: 'ConsumptionCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.purchaseOrder', {
            url: 'purchaseOrder',
            templateUrl: 'purchaseOrder/purchaseOrder.html',
            controller: 'PurchaseOrderCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.messages', {
            url: 'messages',
            templateUrl: 'message/message.html',
            controller: 'MessageCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.vendor', {
            url: 'catalog',
            templateUrl: 'vendor/vendor.html',
            controller: 'VendorCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.ndc', {
            url: 'ndc',
            templateUrl: 'ndc/ndcData.html',
            controller: 'NdcDataCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        })
        .state('root.ftpLog', {
            url: 'ftpLog',
            templateUrl: 'ftpLog/ftpLog.html',
            controller: 'FTPLogCtrl',
            loginRequired: true,
            resolve: {
                loggedin: checkLoggedIn
            }
        });

    $urlRouterProvider
        .otherwise('login');
}]);
