/**
 * Created by John on 7/4/2016.
 */

var Sync = require("synchronize");
var winston = require('winston');
var logger = winston.loggers.get('default');
var rp = require("request-promise");
var https = require("https");
var parse = require("xml-parser");

function IO(config) {
    this.config = config;

    this.url = config["par"].url;
    this.uid = config["par"].svcId;
    this.pwd = config["par"].svcPwd;
    this.domain = config["par"].IODomain;
}

IO.prototype.getParValue = function(clinicId, vendorMaterial, callback) {
    var self = this;
    if (!clinicId) throw Error("No clinic");
    Sync.fiber(function(){
        var clinic = clinicId.substring(1);
        var xml = self.formatRequest(clinic, vendorMaterial);
        try {
            var res = Sync.await(self.getParFromService(xml, Sync.defer()));
            //TODO: format res to get just the values
            callback(null, res);
        } catch (err) {
            logger.error(err);
            callback(err,null);
        }
    });

};

IO.prototype.formatRequest = function(clinic, vendorMaterial) {
    var template = '<?xml version="1.0" encoding="utf-8"?>' +
        '<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:int="http://intelliorder.com">' +
        '<soap:Header/>' +
        '<soap:Body>' +
        '<int:IntelliOrderInventoryStatus>' +
        '<int:ServiceUserID>' + this.uid + '</int:ServiceUserID>' +
        '<int:ServicePassword>' + this.pwd + '</int:ServicePassword>' +
        '<int:IODomain>' + this.domain + '</int:IODomain>' +
        '<int:IOUserID>' + clinic.replace(/\d+$/, "") + '</int:IOUserID>' +
        '<int:IOLocationID>' + clinic + '</int:IOLocationID>' +
        '<int:IOProductID>' + vendorMaterial + '</int:IOProductID>' +
        '</int:IntelliOrderInventoryStatus>' +
        '</soap:Body>' +
        '</soap:Envelope>';

    return template;
};

IO.prototype.getParValuePromise = function(clinicId, vendorMaterial, callback) {
    var self = this;
    if (!clinicId) throw Error("No clinic");
    var clinic = clinicId.substring(1);
    var xml = self.formatRequest(clinic, vendorMaterial);
    if (callback) self.getParFromServicePromise(xml, callback);
    else {
        var out = self.getParFromServicePromise(xml);
        return out;
    }
};

IO.prototype.getParFromServicePromise = function(xml, callback) {
    var self = this;
    var host = self.url.substring(self.url.indexOf("//") + 2, self.url.indexOf(".com") + 4);
    var agentOptions;
    var agent;

    agentOptions = {
        host: host
        , port: '443'
        , path: '/'
        , rejectUnauthorized: false
    };

    agent = new https.Agent(agentOptions);
    var options = {
        uri: self.url,
        method: 'POST',
        headers: {
            'User-Agent': 'fmc (node/' + process.version + ')',
            'Content-Type': 'application/soap+xml'
        },
        body: xml,
        data: xml,
        agent: agent,
        resolveWithFullResponse: true
    };
    if (callback) {
        callback(null, (rp(options).promise()));
    }
    else {
        return (rp(options).promise());
    }
};

IO.prototype.getParFromService = function(xml, callback){
    var self = this;
    var host = self.url.substring(self.url.indexOf("//") + 2, self.url.indexOf(".com") + 4);
    var agentOptions;
    var agent;

    agentOptions = {
        host: host
        , port: '443'
        , path: '/'
        , rejectUnauthorized: false
    };

    agent = new https.Agent(agentOptions);
    var options = {
        uri: self.url,
        method: 'POST',
        headers: {
            'User-Agent': 'fmc (node/' + process.version + ')',
            'Content-Type': 'application/soap+xml'
        },
        body: xml,
        data: xml,
        agent: agent,
        resolveWithFullResponse: true
    };

    rp(options)
        .then(function (response) {
            self.processResponse(response, callback);
        })
        .catch(function (err) {
            // API call failed...
            var obj = {
                minVal: 0,
                maxVal: 0,
                qoh: 0
            };
            callback(null, obj);
        });

};

IO.prototype.processResponse = function(response, callback) {
    //console.log('User has photo %s', photo.toString());
    var bytes = response.body;
    var obj = {
        minVal: 0,
        maxVal: 0,
        qoh: 0,
        vendorMaterial: null
    };

    if (bytes.indexOf("<IntelliOrderInventoryStatusResult>") > 0) {
        //<IntelliOrderInventoryStatusResult>&lt;IntelliOrderInventoryStatus ResponseCode="200" ResponseMessage="Data Retrieval Success." IODomain="FMC" IOUserID="4998BM" IOLocationID="4998BM" IOProductID="36583" QOH="2" Min="0" Max="0" /></IntelliOrderInventoryStatusResult>
        //<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><IntelliOrderInventoryStatusResponse xmlns="http://intelliorder.com"><IntelliOrderInventoryStatusResult>&lt;IntelliOrderInventoryStatus ResponseCode="501" ResponseMessage="IOProductID not found." Domain="FMC" UserID="1008BM" LocationID="1008BM" ItemID="478854" /&gt;</IntelliOrderInventoryStatusResult></IntelliOrderInventoryStatusResponse></soap:Body></soap:Envelope>
        bytes = bytes.replace("&lt;", "<");
        bytes = bytes.replace("&gt;", ">");
        var dataJS = parse(bytes);
        var resNode = dataJS.root.children[0].children[0].children[0].children[0];
        var respCode = resNode.attributes["ResponseCode"];
        //TODO: PArse the response and send back just what we need
        if (respCode == "200") {
            obj.minVal = resNode.attributes["Min"];
            obj.maxVal = resNode.attributes["Max"];
            obj.qoh = resNode.attributes["QOH"];
            obj.vendorMaterial = resNode.attributes["IOProductID"];
        }
    }
    if (callback) callback(null,obj);
    else return obj;
};

module.exports = IO;